@@ -1,6 +1,6 @@
 ;;; dap-mode.el --- Debug Adapter Protocol mode      -*- lexical-binding: t; -*-
 
-;; Copyright (C) 2019-2024  emacs-lsp maintainers
+;; Copyright (C) 2019-2025  emacs-lsp maintainers
 
 ;; This program is free software; you can redistribute it and/or modify
 ;; it under the terms of the GNU General Public License as published by
@@ -18,9 +18,9 @@
 ;; Author: Ivan Yonchovski <yyoncho@gmail.com>
 ;; Keywords: languages, debug
 ;; URL: https://github.com/emacs-lsp/dap-mode
-;; Package-Requires: ((emacs "27.1") (dash "2.18.0") (lsp-mode "6.0") (bui "1.1.0") (f "0.20.0") (s "1.12.0") (lsp-treemacs "0.1") (posframe "0.7.0") (ht "2.3") (lsp-docker "1.0.0"))
-;; Package-Version: 0.8
-;; Package-Revision: 71c2e5a3734d
+;; Package-Requires: ((emacs "28.1") (dash "2.18.0") (lsp-mode "6.0") (bui "1.1.0") (f "0.20.0") (s "1.12.0") (lsp-treemacs "0.1") (posframe "0.7.0") (ht "2.3") (lsp-docker "1.0.0"))
+;; Package-Version: 20251004.452
+;; Package-Revision: 6c74027e39fc
 
 ;;; Commentary:
 ;; Debug Adapter Protocol client for Emacs.
@@ -121,7 +121,7 @@ also `dap--make-terminal-buffer'."
 (defcustom dap-output-buffer-filter '("stdout" "stderr")
   "If non-nil, a list of output types to display in the debug output buffer."
   :group 'dap-mode
-  :type 'list)
+  :type '(repeat string))
 
 (defcustom dap-label-output-buffer-category nil
   "If non-nil, content that is printed to the output buffer will be labelled
@@ -456,8 +456,11 @@ This is in contrast to merely setting it to 0."
       "Failed to persist file: %S"
     (make-directory (file-name-directory file) t)
     (with-temp-file file
-      (erase-buffer)
-      (insert (prin1-to-string to-persist)))))
+      ;; Starting with Emacs 29, `prin1' takes an optional third argument
+      ;; which removes the need for these default bindings.
+      (let ((print-length nil)
+            (print-level nil))
+        (prin1 to-persist (current-buffer))))))
 
 (defun dap--set-sessions (debug-sessions)
   "Update list of debug sessions for WORKSPACE to DEBUG-SESSIONS."
@@ -788,12 +791,13 @@ will be reversed."
       (dap--send-message
        (dap--make-request "source" (list :sourceReference sourceReference))
        (dap--resp-handler
-        (-lambda ((&hash "body" (&hash "content" content)))
-          (switch-to-buffer (generate-new-buffer sourceReferenceKey))
-          (insert content)
-          (goto-char (point-min))
-          (forward-line (1- line))
-          (forward-char column))
+        (-lambda ((&hash "body" body))
+          (let ((content (if body (gethash "content" body) "")))
+            (switch-to-buffer (generate-new-buffer sourceReferenceKey))
+            (insert content)
+            (goto-char (point-min))
+            (forward-line (1- line))
+            (forward-char column)))
         (lambda (errmsg)
           (message "No source code for %s. Cursor at %s:%s. Error: %s." name line column errmsg)))
        debug-session))))
@@ -918,14 +922,13 @@ an OUTPUT-BODY."
 (defun dap--print-to-output-buffer (debug-session str)
   "Insert content from STR into the output buffer associated with DEBUG-SESSION."
   (with-current-buffer (get-buffer-create (dap--debug-session-output-buffer debug-session))
-    (font-lock-mode t)
-    (setq-local buffer-read-only nil)
-    (if (and (eq (current-buffer) (window-buffer (selected-window)))
-             (not (= (point) (point-max))))
-        (save-excursion
-          (dap--insert-at-point-max str))
-      (dap--insert-at-point-max str))
-    (setq-local buffer-read-only t))
+    (turn-on-font-lock)
+    (let ((inhibit-read-only t))
+      (if (and (eq (current-buffer) (window-buffer))
+               (not (eobp)))
+          (save-excursion
+            (dap--insert-at-point-max str))
+        (dap--insert-at-point-max str))))
   (when (and dap-auto-show-output
              (not (dap--debug-session-output-displayed debug-session)))
     (setf (dap--debug-session-output-displayed debug-session) t)
@@ -1914,7 +1917,7 @@ the new template can be used normally with `dap-debug'"
                              (emacs-lisp-mode)
                              (current-buffer)))
     (goto-char (point-max))
-    (when (s-blank? (buffer-string))
+    (when (bobp) ;; Empty (accessible portion of) buffer.
       (insert ";; Eval Buffer with `M-x eval-buffer' to register the newly created template."))
     (insert
      (format "\n\n(dap-register-debug-template\n  \"%s%s\"\n"
@@ -1923,13 +1926,12 @@ the new template can be used normally with `dap-debug'"
     (insert "  (list ")
     (-let ((column (current-column))
            ((fst snd . rst) debug-args))
-      (insert (format "%s %s" fst (prin1-to-string snd)))
+      (insert (format "%s %S" fst snd))
       (cl-loop for (k v) on rst by #'cddr
-               do (if (not (equal k :program-to-start))
-                      (progn
-                        (insert "\n")
-                        (--dotimes column (insert " "))
-                        (insert (format "%s %s" k (prin1-to-string v)))))))
+               do (unless (eq k :program-to-start)
+                    (insert "\n")
+                    (insert-char ?\s column)
+                    (insert (format "%s %S" k v)))))
     (insert "))"))
   (pop-to-buffer "*DAP Templates*")
   (goto-char (point-max)))
